' Program: BMPX.BAS (v2.0)
' This program translates a B&W bitmap file into the correct format
' for the G12864 or G12032 serial graphics LCD modules. It
' downloads the converted bitmap to the display via the serial port.

' =VERSION 2: Support for the G12032 display added. The program
' =automatically configures for this display when given a file of
' =the appropriate size (120x32, 574 bytes).

' ==INSTRUCTIONS:
' The program is written to run from the DOS command line in the
' format:
'
'              BMPX COM1 2400 <filename>
'
' where COM1 is the com port to use (COM1 or COM2 may be used)
' and 2400 is the baud rate (2400 or 9600 are valid).
' Filename is the DOS filename for a monochrome bitmap (.BMP).
' If the file is not in the current DOS path, the full path must be
' given. When download is complete, the program asks the user whether
' to transfer the image to an EEPROM screen and, if so, prompts for
' the screen number. If the transfer targets screens 0 or 1 (location
' of the text font) the program will prompt 'are you sure?' to
' discourage accidental erasure of the font.

' ===================================================================
'                             HOW IT WORKS
' ===================================================================
' =NOTE: The general comments discuss a 128x64-pixel file, but apply
' =to 120x32-pixel BMPs as well. ====================================
' A 128x64-pixel BMP file is a pixel-for-pixel match for the G12864
' screen. The only problem is that the data is organized differently
' and must be "pivoted" before downloading. The G12864 stores screen
' data as a series of bytes starting at the upper lefthand corner and
' working left-to-right and top-to-bottom in a series of 8-pixel-high
' stripes. A mono BMP file starts at the bottom-right corner and works
' from right to left, bottom-to-top in a series of 1-pixel-high stripes.
' It also adds a 63-byte header to the beginning of the file.
' +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'              _____byte 0
'       top   |  ___byte 1
'       left  | |  _byte 2
'           \ | | |
'      bit0   x x x       addresses increase ---> 127
'      bit1   x x x
'      bit2   x x x     
'      bit3   x x x             SCREEN LAYOUT OF
'      bit4   x x x                  G12864
'      bit5   x x x
'      bit6   x x x
'      bit7   x x x
'      bit0   x x x   --next row of bytes starting at address 128
'      ....    ...
' +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'
'        byte2    byte1    byte0        DATA FORMAT FOR UNCOMPRESSED
'       xxxxxxxx xxxxxxxx xxxxxxxx          128x64 MONO .BMP FILE
'                msb<-lsb         \
'                                  bottom right
' +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
' This program reads the .BMP file into a string (really just an array
' of bytes), then rearranges it into an integer array in the G12864
' layout. Once that's done, it opens the serial port, issues a download
' instruction to the G12864, and sends the 1024 bytes of data. As stored
' in the gxbits() array, 1s are white pixels and 0s are black. Before
' downloading, these are inverted to match the G12864 convention of
' 1=black and 0=white.

' ===SUBS AND DECLARARATIONS===
' BSET sets individual bits in the gxbits() array in accordance with the
' G12864 screen layout. Pix() is an array of bytes used as temporary storage
' for the .BMP file. Gxbits() is an integer array that holds the converted
' G12864 screen image
DECLARE SUB BSET (x AS INTEGER, y AS INTEGER)
DEFINT A-Z
DIM pix(1086) AS STRING * 1         ' File contents.
DIM SHARED gxbits(1024) AS INTEGER  ' Rearranged bitmap.
DIM SHARED XWIDTH AS INTEGER        ' LCD width in pixels (120 or 128).
CONST G12864 = 1086                 ' File size (bytes) for G12864.
CONST G12032 = 574                  ' File size (bytes) for G12032.
CONST TRUE = -1                     ' For IF/THEN statements, as "IF TRUE.."
CONST FALSE = 0

' ===INITIALIZATION AND COMMAND PARSING==
CLEAR gxbits                    ' All bytes to 0 to begin.
PRINT : PRINT : PRINT "Graphics conversion/downloading utility"
PRINT "Scott Edwards Electronics Inc. -- www.seetron.com"
file$ = COMMAND$                ' Get the command-line argument.
' COMMAND$ strips any extra lefthand spaces and converts all input
' to uppercase. Extra spaces _between_ parameters are not removed.
space = INSTR(file$, " ")       ' Find 1st space character.
port$ = LEFT$(file$, space)     ' Up to 1st space is com port (1 or 2).
port$ = RTRIM$(port$)           ' Trim extra spaces.
theRest = LEN(file$) - space    ' Remove port spec from file$--everything
file$ = RIGHT$(file$, theRest)  ' from 1st space to end is now file$.

' Check for valid port selection and bail out with error message if bad.
IF port$ <> "COM1" AND port$ <> "COM2" THEN GOTO badPort
space = INSTR(file$, " ")       ' Find 1st space in shortened file$.
baud$ = LEFT$(file$, space)     ' Up to 1st space = baud rate (2400 or 9600).
baud$ = RTRIM$(baud$)           ' Trim extra spaces.
theRest = LEN(file$) - space    ' Remove baud spec from file$--everything
file$ = RIGHT$(file$, theRest)  ' from 1st space to end is now file$.
file$ = LTRIM$(RTRIM$(file$))   ' Remove extraneous spaces from filespec.

' Check for valid baud selection and bail out with error message if bad.
IF baud$ <> "2400" AND baud$ <> "9600" THEN GOTO badBaud
dot = INSTR(file$, ".")         ' Find "." that separates name/extension.
IF dot = 0 THEN                 ' If no dot, then no extension given.
  file$ = file$ + ".BMP"        ' Default extension is "BMP"
END IF

OPEN file$ FOR BINARY AS #1     ' Input the BMP file to pix array.
FILESIZE = LOF(1)               ' Get the file size in bytes.

SELECT CASE FILESIZE    ' Use file size to determine target display.
  CASE G12864           ' File size indicates G12864 128x64 graphic.
    YCOUNT = 63
    MAXBYTE = 1023
    MAXPAGE = 15
    SERACK = TRUE
    XWIDTH = 128
  CASE G12032           ' File size indicates G12032 120x32 graphic.
    YCOUNT = 31
    MAXBYTE = 479
    MAXPAGE = 7
    SERACK = FALSE
    XWIDTH = 120
  CASE ELSE             ' File size does not match any available display.
    PRINT "Incorrect file size. Download cancelled."
    CLOSE
    END
END SELECT

FOR i = 1 TO FILESIZE   ' Input the file contents to array.
     GET #1, i, pix(i - 1)
NEXT i
CLOSE #1
' Copy the .BMP data to the gxbits() array using the BSET subroutine
' to rearrange the bits into the correct layout.
pointer = 62                    ' Point to address after header.
FOR j = YCOUNT TO 0 STEP -1     ' For each address in the array...
  FOR i = 0 TO 15
   temp = INT(ASC(pix(pointer)))
   IF temp AND 128 THEN CALL BSET(((i * 8)), j)         ' Set appropriate
   IF temp AND 64 THEN CALL BSET(((i * 8) + 1), j)      ' bit in the
   IF temp AND 32 THEN CALL BSET(((i * 8) + 2), j)      ' gxbits() array.
   IF temp AND 16 THEN CALL BSET(((i * 8) + 3), j)
   IF temp AND 8 THEN CALL BSET(((i * 8) + 4), j)
   IF temp AND 4 THEN CALL BSET(((i * 8) + 5), j)
   IF temp AND 2 THEN CALL BSET(((i * 8) + 6), j)
   IF temp AND 1 THEN CALL BSET(((i * 8) + 7), j)
   pointer = pointer + 1
  NEXT
NEXT
' Now gxbits() contains a properly formatted copy of the BMP graphic.
' Open the specified serial port at the requested rate and download
' the data to the display.
OPEN port$ + ":" + baud$ + ",N,8,1,CD0,CS0,DS0,OP0" FOR RANDOM AS #1
PRINT #1, CHR$(27); "DG";       ' Start the download.
FOR i = 0 TO MAXBYTE            ' Send gxbits.
  PRINT #1, CHR$(gxbits(i) XOR 255);    ' Send each byte (inverted).
NEXT

' The G12864 sends back a character to acknowledge downloads. The G12032
' does not. So if the file size indicates that a G12864 is the target of
' the download, the code below looks for a response. If the download is
' not for the G12864, response-checking is skipped.
IF SERACK THEN                  ' Serial acknowledgement expected.
  loops = 0                     ' Download done. Look for response.
hold:
  reply$ = INPUT$(LOC(1), #1)   ' Get bytes (if any) from serial-in buffer.
  loops = loops + 1             ' Count the number of attempts
  IF reply$ = "" AND loops < 1000 THEN GOTO hold  ' No response--try again.
  IF reply$ = "" THEN GOTO noResponse     ' 1000 tries, no response=Error.
  PRINT : PRINT "Image successfully downloaded."
  INPUT "Transfer image to EEPROM (Y/N)?", YN$
  IF UCASE$(YN$) <> "Y" THEN
    CLOSE : END
  END IF
ELSE                            ' No serial acknowledgement expected.
  PRINT : PRINT "Image downloaded."
  INPUT "Transfer image to EEPROM (Y/N)?", YN$
  IF UCASE$(YN$) <> "Y" THEN
    CLOSE : END
  END IF
END IF

' Download is complete. Transfer screen to EEPROM?
reEnter:
PRINT "EEPROM screen #, 0 -"; MAXPAGE; : INPUT EEscreen
IF EEscreen > MAXPAGE OR EEscreen < 0 THEN GOTO reEnter
IF EEscreen <= 1 THEN
  INPUT "Overwrite text font (Y/N)?", YN$
  IF UCASE$(YN$) <> "Y" THEN
    CLOSE : END
  END IF
END IF
EEscr$ = LTRIM$(RTRIM$(STR$(EEscreen)))
PRINT #1, CHR$(27); "X"; EEscr$; " ";   ' Request the transfer.
CLOSE                                   ' Close the serial port.
END               ' FINISHED.

' ===================================================================
'                           ERROR MESSAGES
' ===================================================================
badPort:
  PRINT : PRINT "Download cancelled. Port must be COM1 or COM2."
  END

badBaud:
  PRINT : PRINT "Download cancelled. Baud must be 2400 or 9600."
  END

noResponse:
  PRINT : PRINT "Display not responding. Check port, cable, baud rate."
  PRINT "Make sure that config switch 4 is set to ESC, not ctl-Z."
  END

SUB BSET (x AS INTEGER, y AS INTEGER)
' --This sub accepts an xy coordinate and sets the corresponding bit
' of the array gxbits() in accordance with the layout of the G12864
' screen. The routine is structured this way because the program was
' originally tested and debugged using the QBASIC instruction PSET
' to view the results on the PC screen.
  index = x + XWIDTH * (y \ 8)                  ' Determine byte address.
  theBit = 2 ^ (y MOD 8)                        ' Determine bit to be set.
  gxbits(index) = gxbits(index) OR theBit       ' Combine bit with byte.
END SUB

